package uoc.project.myentities.controller;

import java.util.List;
import java.util.stream.Collectors;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import uoc.project.myentities.exceptions.ResourceNotFoundException;
import uoc.project.myentities.mapper.AccountMapper;
import uoc.project.myentities.model.Account;
import uoc.project.myentities.service.AccountService;

/**
 * View controller from Account
 * @author xribes13
 *
 */
@RestController
@RequestMapping("/account")
public class AccountController {

	private static final Logger log = LoggerFactory.getLogger(AccountController.class);

	@Autowired
	private AccountService accountService;

	@Autowired
	private AccountMapper accountMapper;

	@GetMapping
	public List<Account> getAccounts() throws ResourceNotFoundException {
		log.info("Getting all accounts");
		List<Account> accounts = accountService.getAll().stream()
				.map(account -> accountMapper.accountEntityToAccount(account)).collect(Collectors.toList());
		return accounts;
	}

	@GetMapping(value = "{id}")
	public Account getOne(@PathVariable long id) throws ResourceNotFoundException {
		log.info("Getting account with id {}", id);
		return accountMapper.accountEntityToAccount(accountService.getOne(id));
	}

	@PostMapping
	public void add(@RequestBody Account account) {
		log.info("Adding new account");
		accountService.add(accountMapper.accountToAccountEntity(account));
	}

	@PutMapping("{id}")
	public void update(@PathVariable long id, @RequestBody Account account) {
		log.info("Updating account with id {}", id);
		accountService.update(accountMapper.accountToAccountEntity(account));
	}

	@DeleteMapping("{id}")
	public void delete(@PathVariable long id) throws ResourceNotFoundException {
		log.info("Deleting account with id {}", id);
		accountService.delete(id);
	}

}
