package uoc.project.myentities.controller;

import java.util.List;
import java.util.stream.Collectors;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import uoc.project.myentities.core.CoreService;
import uoc.project.myentities.exceptions.ResourceNotFoundException;
import uoc.project.myentities.mapper.ApplicationMapper;
import uoc.project.myentities.model.Application;
import uoc.project.myentities.service.ApplicationService;

/**
 * View controller from Application
 * @author xribes13
 *
 */
@RestController
@RequestMapping("/application")
public class ApplicationController {

	private static final Logger log = LoggerFactory.getLogger(ApplicationController.class);

	@Autowired
	private ApplicationService applicationService;

	@Autowired
	private ApplicationMapper applicationMapper;
	
	@Autowired
	private CoreService coreService;

	@GetMapping
	public List<Application> getApplications() throws ResourceNotFoundException {
		log.info("Getting all applications");
		List<Application> applications = applicationService.getAll().stream()
				.map(application -> applicationMapper.applicationEntityToApplication(application)).collect(Collectors.toList());
		return applications;
	}

	@GetMapping(value = "{id}")
	public Application getOne(@PathVariable long id) throws ResourceNotFoundException {
		log.info("Getting application with id {}", id);
		return applicationMapper.applicationEntityToApplication(applicationService.getOne(id));
	}

	@PostMapping
	public void add(@RequestBody Application application) {
		log.info("Adding new application");
		applicationService.add(applicationMapper.applicationToApplicationEntity(application));
	}

	@PutMapping("{id}")
	public void update(@PathVariable long id, @RequestBody Application application) {
		log.info("Updating application with id {}", id);
		applicationService.update(applicationMapper.applicationToApplicationEntity(application));
	}

	@DeleteMapping("{id}")
	public void delete(@PathVariable long id) throws ResourceNotFoundException {
		log.info("Deleting application with id {}", id);
		coreService.deleteApplication(id);
	}

}
