package uoc.project.myentities.controller;

import java.sql.SQLException;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import uoc.project.myentities.core.CoreService;
import uoc.project.myentities.exceptions.ResourceNotFoundException;
import uoc.project.myentities.mapper.AccountMapper;
import uoc.project.myentities.model.Account;

/**
 * Core view controller
 * @author xribes13
 *
 */
@RestController
@RequestMapping("/core")
public class CoreController {
	
	@Autowired
	private CoreService service;
	
	@Autowired 
	private AccountMapper accountMapper;

	private static final Logger log = LoggerFactory.getLogger(CoreController.class);

	@GetMapping("/load")
	public void load(@RequestParam(name = "entity") String entity, @RequestParam(name="application") String application) {
		log.info("Loading data for " + entity + " and application " + application);
		try {
			service.load(entity, application);
		} catch (SQLException | ResourceNotFoundException e) {
			log.error(e.getLocalizedMessage());
		}
	}
	
	@PostMapping("/synchronize/account")
	public void synchronizeAccount(@RequestBody Account account) {	
		log.info("Synchronizing data for " + account.getName() + " and application " + account.getApplication().getName());
		try {
			service.add(accountMapper.accountToAccountEntity(account));
		} catch (SQLException e) {
			log.error(e.getLocalizedMessage());
		}
	}

}
