package uoc.project.myentities.controller;

import java.util.List;
import java.util.stream.Collectors;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import uoc.project.myentities.exceptions.ResourceNotFoundException;
import uoc.project.myentities.mapper.PermissionMapper;
import uoc.project.myentities.model.Permission;
import uoc.project.myentities.service.PermissionService;

/**
 * View controller from Permission
 * @author xribes13
 *
 */
@RestController
@RequestMapping("/permission")
public class PermissionController {

	private static final Logger log = LoggerFactory.getLogger(PermissionController.class);

	@Autowired
	private PermissionService permissionService;

	@Autowired
	private PermissionMapper permissionMapper;

	@GetMapping
	public List<Permission> getPermissions() throws ResourceNotFoundException {
		log.info("Getting all permissions");
		List<Permission> permissions = permissionService.getAll().stream()
				.map(permission -> permissionMapper.permissionEntityToPermission(permission)).collect(Collectors.toList());
		return permissions;
	}

	@GetMapping(value = "{id}")
	public Permission getOne(@PathVariable long id) throws ResourceNotFoundException {
		log.info("Getting permission with id {}", id);
		return permissionMapper.permissionEntityToPermission(permissionService.getOne(id));
	}

//	@PostMapping
//	public void add(@RequestBody Permission permission) {
//		log.info("Adding new permission");
//		permissionService.add(permissionMapper.permissionToPermissionEntity(permission));
//	}
//
//	@PutMapping("{id}")
//	public void update(@PathVariable long id, @RequestBody Permission permission) {
//		log.info("Updating permission with id {}", id);
//		permissionService.update(permissionMapper.permissionToPermissionEntity(permission));
//	}
//
//	@DeleteMapping("{id}")
//	public void delete(@PathVariable long id) throws ResourceNotFoundException {
//		log.info("Deleting permission with id {}", id);
//		permissionService.delete(id);
//	}

}
