package uoc.project.myentities.controller;

import java.util.List;
import java.util.stream.Collectors;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import uoc.project.myentities.entities.UserEntity;
import uoc.project.myentities.exceptions.ResourceNotFoundException;
import uoc.project.myentities.mapper.UserMapper;
import uoc.project.myentities.model.User;
import uoc.project.myentities.service.UserService;

/**
 * View controller from User
 * @author xribes13
 *
 */
@RestController
@RequestMapping("/user")
public class UserController {

	private static final Logger log = LoggerFactory.getLogger(UserController.class);

	@Autowired
	private UserService userService;

	@Autowired
	private UserMapper userMapper;

	@GetMapping
	public List<User> getUsers() throws ResourceNotFoundException {
		log.info("Getting all users");
		List<User> users = userService.getAll().stream().map(userEntity -> userMapper.userEntityToUser(userEntity))
				.collect(Collectors.toList());
		return users;
	}

	@GetMapping(value = "{id}")
	public User getOne(@PathVariable long id) throws ResourceNotFoundException {
		log.info("Getting user with id {}",id);
		return userMapper.userEntityToUser(userService.getOne(id));
	}

	@PostMapping
	public User add(@RequestBody User user){
		log.info("Adding new user");
		UserEntity entity = userService.add(userMapper.userToUserEntity(user));
		return userMapper.userEntityToUser(entity);
	}

	@PutMapping("{id}")
	public User update(@PathVariable long id, @RequestBody User user) {
		log.info("Updating user with id {}", id);
		UserEntity entity = userService.update(userMapper.userToUserEntity(user));
		return userMapper.userEntityToUser(entity);
	}

	@DeleteMapping("{id}")
	public void delete(@PathVariable long id) throws ResourceNotFoundException {
		log.info("Deleting user with id {}", id);
		userService.delete(id);
	}

}
