package uoc.project.myentities.core.impl;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.List;
import java.util.Optional;
import java.util.stream.Collectors;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import uoc.project.myentities.core.CoreService;
import uoc.project.myentities.core.connectors.MySQLConnector;
import uoc.project.myentities.entities.AccountEntity;
import uoc.project.myentities.entities.ApplicationEntity;
import uoc.project.myentities.entities.ApplicationQueryEntity;
import uoc.project.myentities.entities.PermissionEntity;
import uoc.project.myentities.enums.ActionEnum;
import uoc.project.myentities.enums.EntityEnum;
import uoc.project.myentities.enums.TypeEnum;
import uoc.project.myentities.exceptions.ResourceNotFoundException;
import uoc.project.myentities.service.AccountService;
import uoc.project.myentities.service.ApplicationService;
import uoc.project.myentities.service.PermissionService;
import uoc.project.myentities.service.UserService;

@Service
public class CoreServiceImpl implements CoreService {

	@Autowired
	private ApplicationService applicationService;

	@Autowired
	private AccountService accountService;

	@Autowired
	private UserService userService;

	@Autowired
	private PermissionService permissionService;

	@Autowired
	private MySQLConnector sqlConnector;

	private static final Logger log = LoggerFactory.getLogger(CoreServiceImpl.class);

	@Override
	public void load(String entityType, String applicationId) throws SQLException, ResourceNotFoundException {

		ApplicationEntity application = null;
		List<ApplicationQueryEntity> queries = null;
		try {
			application = applicationService.getOne(Long.valueOf(applicationId));
			if ("ACCOUNT".equals(entityType)) {
				queries = application.getQueries().stream().filter(query -> EntityEnum.ACCOUNT.equals(query.getEntity())
						&& ActionEnum.SELECT_ALL.equals(query.getAction())).collect(Collectors.toList());
			}

			if ("ROLE".equals(entityType)) {
				queries = application.getQueries().stream().filter(query -> EntityEnum.ROLE.equals(query.getEntity())
						&& ActionEnum.SELECT_ALL.equals(query.getAction())).collect(Collectors.toList());
			}

			if ("PERMISSION".equals(entityType)) {
				queries = application.getQueries().stream()
						.filter(query -> EntityEnum.PERMISSION.equals(query.getEntity())
								&& ActionEnum.SELECT_ALL.equals(query.getAction()))
						.collect(Collectors.toList());
			}

			if (TypeEnum.MySQL.equals(application.getType())) {
				Connection connect = sqlConnector.getConnection(application.getHost(), application.getUser(),
						application.getPassword());

				if (connect != null && !queries.isEmpty()) {
					Statement stmt = connect.createStatement();
					ResultSet rs = stmt.executeQuery(queries.get(0).getQuery());

					while (rs.next()) {
						log.info("Found entity with id " + rs.getInt(1));
						if (EntityEnum.ACCOUNT.equals(queries.get(0).getEntity())) {
							AccountEntity account = accountService.getByExternalId(rs.getLong(1));
							if (account == null) {
								account = new AccountEntity();
							}
							account.setExternalId(rs.getLong(1));
							account.setApplication(application);
							// USER - UNKNOWN ALWAYS SHOULD EXIST
							account.setUser(userService.getOne(1));
							account.setName(rs.getString(2));
							account.setDescription(rs.getString(3));
							accountService.add(account);
						}
						if (EntityEnum.ROLE.equals(queries.get(0).getEntity())) {
							PermissionEntity permission = permissionService.getByExternalId(rs.getLong(1));
							if (permission == null) {
								permission = new PermissionEntity();
							}
							permission.setExternalId(rs.getLong(1));
							permission.setApplication(application);
							permission.setName(rs.getString(2));
							permission.setDescription(rs.getString(3));
							permissionService.add(permission);
						}
						if (EntityEnum.PERMISSION.equals(queries.get(0).getEntity())) {

						}
					}

					connect.close();
				}
			}
		} catch (NumberFormatException e) {
			log.error(e.getLocalizedMessage());
		}

	}

	@Override
	public void deleteApplication(long applicationId) {
		// TODO Auto-generated method stub

	}

	@Override
	public void add(Object entity) throws SQLException {
		ApplicationEntity application = null;
		String queryToExecute = null;
		if (entity instanceof AccountEntity) {
			AccountEntity account = ((AccountEntity) entity);
			application = account.getApplication();
			if (account.getExternalId() != 0) {
				Optional<ApplicationQueryEntity> query = application.getQueries().stream().filter(
						q -> EntityEnum.ACCOUNT.equals(q.getEntity()) && ActionEnum.UPDATE.equals(q.getAction()))
						.findFirst();
				if (query.isPresent()) {
					queryToExecute = query.get().getQuery();
					queryToExecute = queryToExecute.replaceFirst("\\?", account.getName());
					queryToExecute = queryToExecute.replaceFirst("\\?", account.getDescription());
					queryToExecute = queryToExecute.replaceFirst("\\?", String.valueOf(account.getExternalId()));
				}
			} else {
				Optional<ApplicationQueryEntity> query = application.getQueries().stream().filter(
						q -> EntityEnum.ACCOUNT.equals(q.getEntity()) && ActionEnum.INSERT.equals(q.getAction()))
						.findFirst();
				if (query.isPresent()) {
					queryToExecute = query.get().getQuery();
					queryToExecute = queryToExecute.replaceFirst("\\?", account.getName());
					queryToExecute = queryToExecute.replaceFirst("\\?", account.getDescription());
				}
			}
		}
		if (entity instanceof PermissionEntity) {

		}
		
		if(application != null && queryToExecute != null) {
			log.info("Executing query... " + queryToExecute);
			Connection connect = sqlConnector.getConnection(application.getHost(), application.getUser(),
					application.getPassword());
			if(connect != null) {
				Statement stmt = connect.createStatement();
				stmt.executeUpdate(queryToExecute, Statement.RETURN_GENERATED_KEYS);
		        ResultSet rs = stmt.getGeneratedKeys();
		        long externalId = 0L;
		        if (rs.next()){
		        	externalId =rs.getInt(1);
		        }
				
		        if(entity instanceof AccountEntity) {
		        	((AccountEntity) entity).setExternalId(externalId);
		        	accountService.add(((AccountEntity) entity));
		        }
			}
		}
	}
	
}
