package uoc.project.myentities.mapper;

import java.util.List;
import java.util.stream.Collectors;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import uoc.project.myentities.entities.ApplicationEntity;
import uoc.project.myentities.entities.ApplicationQueryEntity;
import uoc.project.myentities.model.Application;
import uoc.project.myentities.model.ApplicationQuery;

@Component
public class ApplicationMapper {

	@Autowired
	private QueryMapper queryMapper;

	/**
	 * Converts a @application into a @ApplicationEntity object
	 * @param application
	 * @return
	 */
	public ApplicationEntity applicationToApplicationEntity(Application application) {
		ApplicationEntity applicationEntity = new ApplicationEntity();
		applicationEntity.setId(application.getId());
		applicationEntity.setHost(application.getHost());
		applicationEntity.setName(application.getName());
		applicationEntity.setDescription(application.getDescription());
		applicationEntity.setUser(application.getUser());
		applicationEntity.setPassword(application.getPassword());
		applicationEntity.setType(application.getType());
		List<ApplicationQueryEntity> queries = application.getQueries().stream()
				.map(query -> queryMapper.appQueryToAppQueryEntity(query)).collect(Collectors.toList());
		applicationEntity.setQueries(queries);
		return applicationEntity;
	}

	/**
	 * Converts a @ApplicationEntity into a @Application object
	 * @param applicationEntity
	 * @return
	 */
	public Application applicationEntityToApplication(ApplicationEntity applicationEntity) {
		Application application = new Application();
		application.setId(applicationEntity.getId());
		application.setHost(applicationEntity.getHost());
		application.setName(applicationEntity.getName());
		application.setDescription(applicationEntity.getDescription());
		application.setUser(applicationEntity.getUser());
		application.setPassword(applicationEntity.getPassword());
		application.setType(applicationEntity.getType());
		List<ApplicationQuery> queries = applicationEntity.getQueries().stream()
				.map(query -> queryMapper.appQueryEntityToAppQuery(query)).collect(Collectors.toList());
		application.setQueries(queries);
		return application;
	}
}
