package uoc.project.myentities.service;

import java.util.Collection;

import uoc.project.myentities.entities.AccountEntity;
import uoc.project.myentities.exceptions.ResourceNotFoundException;

public interface AccountService {

	/**
	 * Get all @AccountEntity
	 * @return
	 * @throws ResourceNotFoundException
	 */
	Collection<AccountEntity> getAll() throws ResourceNotFoundException;
	
	/**
	 * Get all @AccountEntity from an applicationId
	 * @param applicationId
	 * @return
	 * @throws ResourceNotFoundException
	 */
	Collection<AccountEntity> getAllByApplication(long applicationId) throws ResourceNotFoundException;

	/**
	 * Get all @AccountEntity from a userId
	 * @param userId
	 * @return
	 * @throws ResourceNotFoundException
	 */
	Collection<AccountEntity> getAllByUser(long userId) throws ResourceNotFoundException;
	
	/**
	 * Get an @AccountEntity by its id
	 * @param id
	 * @return
	 * @throws ResourceNotFoundException
	 */
	AccountEntity getOne(long id) throws ResourceNotFoundException;
	
	/**
	 * Gets an @AccountEntity by its externalId field
	 * @param externalId
	 * @return
	 * @throws ResourceNotFoundException
	 */
	AccountEntity getByExternalId(long externalId) throws ResourceNotFoundException;

	/**
	 * Creates a new @AccountEntity
	 * @param user
	 * @return
	 */
	AccountEntity add(AccountEntity user);

	/**
	 * Updates an existing @AccountEntity
	 * @param user
	 * @return
	 */
	AccountEntity update(AccountEntity user);

	/**
	 * Delete an existing @AccountEntity by its id
	 * @param id
	 * @throws ResourceNotFoundException
	 */
	void delete(long id) throws ResourceNotFoundException;

}
