package uoc.project.myentities.service.impl;

import java.util.Collection;
import java.util.List;
import java.util.Optional;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Example;
import org.springframework.stereotype.Service;

import uoc.project.myentities.entities.AccountEntity;
import uoc.project.myentities.entities.PermissionAccountEntity;
import uoc.project.myentities.exceptions.ResourceNotFoundException;
import uoc.project.myentities.repository.AccountRepository;
import uoc.project.myentities.service.AccountService;
import uoc.project.myentities.service.ApplicationService;
import uoc.project.myentities.service.PermissionService;

@Service
public class AccountServiceImpl implements AccountService {

	@Autowired
	private AccountRepository accountRepository;

	@Autowired
	private ApplicationService applicationService;

	@Autowired
	private PermissionService permissionService;

	@Override
	public Collection<AccountEntity> getAll() throws ResourceNotFoundException {
		List<AccountEntity> entities = accountRepository.findAll();
		if (entities.isEmpty()) {
			throw new ResourceNotFoundException("", "", "");
		}
		return entities;
	}

	@Override
	public Collection<AccountEntity> getAllByApplication(long applicationId) throws ResourceNotFoundException {
		AccountEntity example = new AccountEntity();
		example.setApplication(applicationService.getOne(applicationId));

		List<AccountEntity> entities = accountRepository.findAll(Example.of(example));
		if (entities.isEmpty()) {
			throw new ResourceNotFoundException("", "", "");
		}
		return entities;
	}

	@Override
	public AccountEntity getOne(long id) throws ResourceNotFoundException {
		AccountEntity entity = accountRepository.getOne(id);
		if (entity == null) {
			throw new ResourceNotFoundException("", "", "");
		}
		return entity;
	}

	@Override
	public AccountEntity getByExternalId(long externalId) throws ResourceNotFoundException {
		AccountEntity example = new AccountEntity();
		example.setExternalId(externalId);

		Optional<AccountEntity> entity = accountRepository.findOne(Example.of(example));
		if (!entity.isPresent()) {
			return null;
		}
		return entity.get();
	}

	@Override
	public Collection<AccountEntity> getAllByUser(long userId) throws ResourceNotFoundException {
		return accountRepository.findAccountsByUserId(userId);
	}

	@Override
	public AccountEntity add(AccountEntity entity) {
		return accountRepository.save(entity);
	}

	@Override
	public AccountEntity update(AccountEntity entity) {
		return accountRepository.save(entity);
	}

	@Override
	public void delete(long id) throws ResourceNotFoundException {
		AccountEntity entity = accountRepository.getOne(id);
		if (entity == null) {
			throw new ResourceNotFoundException("", "", "");
		}

		List<PermissionAccountEntity> accountPermissions = (List<PermissionAccountEntity>) permissionService
				.getPermissionsByAccount(id);
		accountPermissions.stream().forEach(permission -> {
			try {
				permissionService.deletePermissionAccount(permission.getId());
			} catch (ResourceNotFoundException e) {
				e.printStackTrace();
			}
		});

		accountRepository.delete(entity);
	}

}
