package uoc.project.myentities.service.impl;

import java.util.Collection;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import uoc.project.myentities.entities.AccountEntity;
import uoc.project.myentities.entities.UserEntity;
import uoc.project.myentities.exceptions.ResourceNotFoundException;
import uoc.project.myentities.repository.UserRepository;
import uoc.project.myentities.service.AccountService;
import uoc.project.myentities.service.UserService;

@Service
public class UserServiceImpl implements UserService {

	@Autowired
	private UserRepository userRepository;

	@Autowired
	private AccountService accountService;

	@Override
	public Collection<UserEntity> getAll() throws ResourceNotFoundException {
		List<UserEntity> entities = userRepository.findAll();
		if (entities.isEmpty()) {
			throw new ResourceNotFoundException("", "", "");
		}
		return entities;
	}

	@Override
	public UserEntity getOne(long id) throws ResourceNotFoundException {
		UserEntity entity = userRepository.getOne(id);
		if (entity == null) {
			throw new ResourceNotFoundException("", "", "");
		}
		return entity;
	}

	@Override
	public UserEntity add(UserEntity user) {
		return userRepository.save(user);
	}

	@Override
	public UserEntity update(UserEntity user) {
		return userRepository.save(user);
	}

	@Override
	public void delete(long id) throws ResourceNotFoundException {
		UserEntity entity = userRepository.getOne(id);
		if (entity == null) {
			throw new ResourceNotFoundException("", "", "");
		}

		List<AccountEntity> userAccounts = (List<AccountEntity>) accountService.getAllByUser(id);
		userAccounts.stream().forEach(account -> {
			try {
				accountService.delete(account.getId());
			} catch (ResourceNotFoundException e) {
				e.printStackTrace();
			}
		});

		userRepository.delete(entity);
	}
}
